@@ -455,68 +455,6 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 #define emith_bic_r_imm(r, imm) \
 	emith_arith_r_imm(4, r, ~(imm))
 
-// fake conditionals (using SJMP instead)
-#define emith_move_r_imm_c(cond, r, imm) \
-	emith_move_r_imm(r, imm)
-#define emith_add_r_imm_c(cond, r, imm) \
-	emith_add_r_imm(r, imm)
-#define emith_sub_r_imm_c(cond, r, imm) \
-	emith_sub_r_imm(r, imm)
-#define emith_or_r_imm_c(cond, r, imm) \
-	emith_or_r_imm(r, imm)
-#define emith_eor_r_imm_c(cond, r, imm) \
-	emith_eor_r_imm(r, imm)
-#define emith_eor_r_imm_ptr_c(cond, r, imm) \
-	emith_eor_r_imm_ptr(r, imm)
-#define emith_bic_r_imm_c(cond, r, imm) \
-	emith_bic_r_imm(r, imm)
-#define emith_tst_r_imm_c(cond, r, imm) \
-	emith_tst_r_imm(r, imm)
-#define emith_move_r_r_ptr_c(cond, d, s) \
-	emith_move_r_r_ptr(d, s)
-#define emith_ror_c(cond, d, s, cnt) \
-	emith_ror(d, s, cnt)
-#define emith_and_r_r_c(cond, d, s) \
-	emith_and_r_r(d, s)
-#define emith_add_r_r_imm_c(cond, d, s, imm) \
-	emith_add_r_r_imm(d, s, imm)
-#define emith_sub_r_r_imm_c(cond, d, s, imm) \
-	emith_sub_r_r_imm(d, s, imm)
-
-#define emith_read8_r_r_r_c(cond, r, rs, rm) \
-	emith_read8_r_r_r(r, rs, rm)
-#define emith_read8s_r_r_r_c(cond, r, rs, rm) \
-	emith_read8s_r_r_r(r, rs, rm)
-#define emith_read16_r_r_r_c(cond, r, rs, rm) \
-	emith_read16_r_r_r(r, rs, rm)
-#define emith_read16s_r_r_r_c(cond, r, rs, rm) \
-	emith_read16s_r_r_r(r, rs, rm)
-#define emith_read_r_r_r_c(cond, r, rs, rm) \
-	emith_read_r_r_r(r, rs, rm)
-
-#define emith_read_r_r_offs_c(cond, r, rs, offs) \
-	emith_read_r_r_offs(r, rs, offs)
-#define emith_read_r_r_offs_ptr_c(cond, r, rs, offs) \
-	emith_read_r_r_offs_ptr(r, rs, offs)
-#define emith_write_r_r_offs_c(cond, r, rs, offs) \
-	emith_write_r_r_offs(r, rs, offs)
-#define emith_write_r_r_offs_ptr_c(cond, r, rs, offs) \
-	emith_write_r_r_offs_ptr(r, rs, offs)
-#define emith_read8_r_r_offs_c(cond, r, rs, offs) \
-	emith_read8_r_r_offs(r, rs, offs)
-#define emith_write8_r_r_offs_c(cond, r, rs, offs) \
-	emith_write8_r_r_offs(r, rs, offs)
-#define emith_read16_r_r_offs_c(cond, r, rs, offs) \
-	emith_read16_r_r_offs(r, rs, offs)
-#define emith_write16_r_r_offs_c(cond, r, rs, offs) \
-	emith_write16_r_r_offs(r, rs, offs)
-#define emith_jump_reg_c(cond, r) \
-	emith_jump_reg(r)
-#define emith_jump_ctx_c(cond, offs) \
-	emith_jump_ctx(offs)
-#define emith_ret_c(cond) \
-	emith_ret()
-
 // _r_r_imm - use lea
 #define emith_add_r_r_imm(d, s, imm) do { \
 	if (imm == 0) \
@@ -629,11 +567,6 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 	if (count) emith_and_r_imm(d, t); \
 } while (0)
 
-#define emith_clear_msb_c(cond, d, s, count) do { \
-	(void)(cond); \
-	emith_clear_msb(d, s, count); \
-} while (0)
-
 #define emith_sext(d, s, bits) do { \
 	emith_lsl(d, s, 32 - (bits)); \
 	emith_asr(d, d, 32 - (bits)); \
@@ -858,8 +791,6 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 
 #define emith_ctx_read(r, offs) \
 	emith_read_r_r_offs(r, CONTEXT_REG, offs)
-#define emith_ctx_read_c(cond, r, offs) \
-	emith_ctx_read(r, offs)
 
 #define emith_ctx_read_ptr(r, offs) do { \
 	EMIT_REX_IF(1, r, CONTEXT_REG); \
@@ -978,28 +909,26 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 	emith_ret(); \
 } while (0)
 
+#define emith_abijump(target) \
+	emith_jump(target)
 #define emith_abijump_reg(r) \
 	emith_jump_reg(r)
-#define emith_abijump_reg_c(cond, r) \
-	emith_abijump_reg(r)
 #define emith_abicall(target) \
 	emith_call(target)
-#define emith_abicall_cond(cond, target) \
-	emith_abicall(target)
 #define emith_abicall_reg(r) \
 	emith_call_reg(r)
 
 
-#define EMITH_JMP_START(cond) { \
-	u8 *cond_ptr; \
+#define EMITH_JMP_START(cond) \
+{ 	u8 *cond_ptr; \
 	JMP8_POS(cond_ptr)
 
 #define EMITH_JMP_END(cond) \
 	JMP8_EMIT(cond, cond_ptr); \
 }
 
-#define EMITH_JMP3_START(cond) { \
-	u8 *cond_ptr, *else_ptr; \
+#define EMITH_JMP3_START(cond) \
+{ 	u8 *cond_ptr, *else_ptr; \
 	JMP8_POS(cond_ptr)
 
 #define EMITH_JMP3_MID(cond) \
@@ -1282,12 +1211,12 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 		t3 = rcache_get_reg(reg, RC_GR_RMW, NULL);	\
 		emith_cmp_r_r(t3, t2);				\
 		EMITH_SJMP_START(DCOND_HI);			\
-		emith_sub_r_r_imm_c(DCOND_LS, t2, t3, 1);	\
+		emith_sub_r_r_imm(t2, t3, 1);	                \
 		EMITH_SJMP_END(DCOND_HI);			\
 		/* if (reg <= 1) turns = 0 */			\
 		emith_cmp_r_imm(t3, 1);				\
 		EMITH_SJMP_START(DCOND_HI);			\
-		emith_move_r_imm_c(DCOND_LS, t2, 0);		\
+		emith_move_r_imm(t2, 0);		        \
 		EMITH_SJMP_END(DCOND_HI);			\
 		/* reg -= turns */				\
 		emith_sub_r_r(t3, t2);				\
@@ -1369,11 +1298,11 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 	emith_lsr(rm, mh, 31);                    \
 	emith_addf_r_r(rn, rm);                   \
 	EMITH_SJMP_START(DCOND_EQ); /* sum != 0 -> -ovl */ \
-	emith_move_r_imm_c(DCOND_NE, ml, 0x00000000); \
-	emith_move_r_imm_c(DCOND_NE, mh, 0x00008000); \
+	emith_move_r_imm(ml, 0x00000000);         \
+	emith_move_r_imm(mh, 0x00008000);         \
 	EMITH_SJMP_START(DCOND_MI); /* sum < 0 -> -ovl */ \
-	emith_sub_r_imm_c(DCOND_PL, ml, 1); /* 0xffffffff */ \
-	emith_sub_r_imm_c(DCOND_PL, mh, 1); /* 0x00007fff */ \
+	emith_sub_r_imm(ml, 1); /* 0xffffffff */  \
+	emith_sub_r_imm(mh, 1); /* 0x00007fff */  \
 	EMITH_SJMP_END(DCOND_MI);                 \
 	EMITH_SJMP_END(DCOND_EQ);                 \
 	EMITH_SJMP_END(DCOND_EQ);                 \
@@ -1395,10 +1324,10 @@ enum { xAX = 0, xCX, xDX, xBX, xSP, xBP, xSI, xDI,	// x86-64,i386 common
 	emith_addf_r_r(rn, mh); /* sum = MACH + ((MACL>>31)&1) */ \
 	EMITH_SJMP_START(DCOND_EQ); /* sum != 0 -> overflow */ \
 	/* XXX: LSB signalling only in SH1, or in SH2 too? */ \
-	emith_move_r_imm_c(DCOND_NE, mh, 0x00000001); /* LSB of MACH */ \
-	emith_move_r_imm_c(DCOND_NE, ml, 0x80000000); /* -overflow */ \
+	emith_move_r_imm(mh, 0x00000001); /* LSB of MACH */ \
+	emith_move_r_imm(ml, 0x80000000); /* -overflow */ \
 	EMITH_SJMP_START(DCOND_MI); /* sum > 0 -> +overflow */ \
-	emith_sub_r_imm_c(DCOND_PL, ml, 1); /* 0x7fffffff */ \
+	emith_sub_r_imm(ml, 1); /* 0x7fffffff */  \
 	EMITH_SJMP_END(DCOND_MI);                 \
 	EMITH_SJMP_END(DCOND_EQ);                 \
 	EMITH_SJMP_END(DCOND_EQ);                 \
@@ -1422,7 +1351,7 @@ static void emith_clr_t_cond(int sr)
 static void emith_set_t_cond(int sr, int cond)
 {
   EMITH_SJMP_START(emith_invert_cond(cond));
-  emith_or_r_imm_c(cond, sr, T);
+  emith_or_r_imm(sr, T);
   EMITH_SJMP_END(emith_invert_cond(cond));
 }
 
