@@ -50,7 +50,7 @@ typedef struct SH2_
 #define SH2_STATE_CPOLL (1 << 2)	// polling comm regs
 #define SH2_STATE_VPOLL (1 << 3)	// polling VDP
 #define SH2_STATE_RPOLL (1 << 4)	// polling address in SDRAM
-#define SH2_IN_DRC      (1 << 7)	// DRC in use
+#define SH2_IN_DRC      (1 << 7)	// DRC executing
 	unsigned int	state;
 	uint32_t	poll_addr;
 	unsigned int	poll_cycles;
