@@ -76,9 +76,11 @@ static void dmac_te_irq(SH2 *sh2, struct dma_chan *chan)
 
 static void dmac_transfer_complete(SH2 *sh2, struct dma_chan *chan)
 {
+  u32 a = sh2->poll_addr;
   chan->chcr |= DMA_TE; // DMA has ended normally
 
-  p32x_sh2_poll_event(sh2->poll_addr, sh2, SH2_STATE_SLEEP, SekCyclesDone());
+  // wake SLEEP and CPOLL since xRick has a loop polling on both TE and COMM
+  p32x_sh2_poll_event(a, sh2, SH2_STATE_SLEEP|SH2_STATE_CPOLL, SekCyclesDone());
   if (chan->chcr & DMA_IE)
     dmac_te_irq(sh2, chan);
 }
