@@ -30,6 +30,14 @@ static int handle_mp3(const char *fname, int index)
   fs = ftell(tmp_file);
   fseek(tmp_file, 0, SEEK_SET);
 
+  kBps = mp3_get_bitrate(tmp_file, fs) / 8;
+  if (ret != 0 || kBps <= 0)
+  {
+    elprintf(EL_STATUS, "track %2i: mp3 bitrate %i", index+1, kBps);
+    fclose(tmp_file);
+    return -1;
+  }
+
 #ifdef _PSP_FW_VERSION
   // some systems (like PSP) can't have many open files at a time,
   // so we work with their names instead.
@@ -37,18 +45,6 @@ static int handle_mp3(const char *fname, int index)
   tmp_file = (void *) strdup(fname);
 #endif
 
-  kBps = mp3_get_bitrate(tmp_file, fs) / 8;
-  if (ret != 0 || kBps <= 0)
-  {
-    elprintf(EL_STATUS, "track %2i: mp3 bitrate %i", index+1, kBps);
-#ifdef _PSP_FW_VERSION
-    free(tmp_file);
-#else
-    fclose(tmp_file);
-#endif
-    return -1;
-  }
-
   track->type = CT_MP3;
   track->fd = tmp_file;
   track->offset = 0;
@@ -58,6 +54,39 @@ static int handle_mp3(const char *fname, int index)
   return fs;
 }
 
+static int handle_ogg(const char *fname, int index)
+{
+  track_t *track = &cdd.toc.tracks[index];
+  FILE *tmp_file;
+  int fs;
+
+  tmp_file = fopen(fname, "rb");
+  if (tmp_file == NULL)
+    return -1;
+
+  fs = ogg_get_length(tmp_file);
+
+  if (fs <= 0)
+  {
+    elprintf(EL_STATUS, "track %2i: ogg length %i", index+1, fs);
+    fclose(tmp_file);
+    return -1;
+  }
+
+#ifdef _PSP_FW_VERSION
+  // some systems (like PSP) can't have many open files at a time,
+  // so we work with their names instead.
+  fclose(tmp_file);
+  tmp_file = (void *) strdup(fname);
+#endif
+
+  track->type = CT_OGG;
+  track->fd = tmp_file;
+  track->offset = 0;
+
+  return fs * 75 / 1000;
+}
+
 static void to_upper(char *d, const char *s)
 {
   for (; *s != 0; d++, s++) {
@@ -151,6 +180,12 @@ int load_cd_image(const char *cd_img_name, int *type)
           break;
         length = ret;
       }
+      else if (cue_data->tracks[n].type == CT_OGG) {
+        ret = handle_ogg(cue_data->tracks[n].fname, index);
+        if (ret < 0)
+          break;
+        length = ret;
+      }
       else if (cue_data->tracks[n].fname != NULL)
       {
         pm_file *f = pm_open(cue_data->tracks[n].fname);
