@@ -1,6 +1,7 @@
 /*
  * Memory I/O handlers for Sega/Mega CD.
  * (C) notaz, 2007-2009
+ * (C) irixxxx, 2019-2024
  *
  * This work is licensed under the terms of MAME license.
  * See COPYING file in the top-level directory.
@@ -8,6 +9,7 @@
 
 #include "../pico_int.h"
 #include "../memory.h"
+#include "megasd.h"
 
 uptr s68k_read8_map  [0x1000000 >> M68K_MEM_SHIFT];
 uptr s68k_read16_map [0x1000000 >> M68K_MEM_SHIFT];
@@ -67,7 +69,7 @@ static void remap_word_ram(u32 r3);
 
 // poller detection
 #define POLL_LIMIT 16
-#define POLL_CYCLES 64
+#define POLL_CYCLES 52
 
 void m68k_comm_check(u32 a)
 {
@@ -99,7 +101,7 @@ u32 pcd_stopwatch_read(int sub)
 {
   // ugh... stopwatch runs 384 cycles per step, divide by mult with inverse
   u32 d = sub ? SekCyclesDoneS68k() : pcd_cycles_m68k_to_s68k(SekCyclesDone());
-  d = ((d - Pico_mcd->m.stopwatch_base_c) * ((1LL << 32) / 384)) >> 32;
+  d = DIVQ32(d - Pico_mcd->m.stopwatch_base_c, 384);
   return d & 0x0fff;
 }
 
@@ -121,18 +123,20 @@ static u32 m68k_reg_read16(u32 a)
       elprintf(EL_CDREG3, "m68k_regs r3: %02x @%06x", (u8)d, SekPc);
       goto end;
     case 4:
+      pcd_sync_s68k(SekCyclesDone(), 0);
       d = Pico_mcd->s68k_regs[4]<<8;
       goto end;
     case 6:
-      d = *(u16 *)(Pico.rom + 0x72);
+      d = *(u16 *)(Pico_mcd->bios + 0x72);
       goto end;
     case 8:
-      d = cdc_host_r();
+      d = cdc_host_r(0);
       goto end;
     case 0xa:
       elprintf(EL_UIO, "m68k FIXME: reserved read");
       goto end;
     case 0xc: // 384 cycle stopwatch timer
+      pcd_sync_s68k(SekCyclesDone(), 0);
       d = pcd_stopwatch_read(0);
       elprintf(EL_CDREGS, "m68k stopwatch timer read (%04x)", d);
       goto end;
@@ -142,7 +146,7 @@ static u32 m68k_reg_read16(u32 a)
     // comm flag/cmd/status (0xE-0x2F)
     m68k_comm_check(a);
     d = (Pico_mcd->s68k_regs[a]<<8) | Pico_mcd->s68k_regs[a+1];
-    goto end;
+    return d;
   }
 
   elprintf(EL_UIO, "m68k_regs FIXME invalid read @ %02x", a);
@@ -166,12 +170,14 @@ void m68k_reg_write8(u32 a, u32 d)
   switch (a) {
     case 0:
       d &= 1;
-      Pico_mcd->m.state_flags &= ~PCD_ST_S68K_IFL2;
-      if (d) Pico_mcd->m.state_flags |= PCD_ST_S68K_IFL2;
+      pcd_sync_s68k(SekCyclesDone(), 0);
       if (d && (Pico_mcd->s68k_regs[0x33] & PCDS_IEN2)) {
         elprintf(EL_INTS, "m68k: s68k irq 2");
-        pcd_sync_s68k(SekCyclesDone(), 0);
+        Pico_mcd->m.state_flags |= PCD_ST_S68K_IFL2;
         pcd_irq_s68k(2, 1);
+      } else {
+        Pico_mcd->m.state_flags &= ~PCD_ST_S68K_IFL2;
+        pcd_irq_s68k(2, 0);
       }
       return;
     case 1:
@@ -192,6 +198,7 @@ void m68k_reg_write8(u32 a, u32 d)
         Pico_mcd->m.state_flags &= ~(PCD_ST_S68K_RST|PCD_ST_S68K_POLL|PCD_ST_S68K_SLEEP);
         elprintf(EL_CDREGS, "m68k: resetting s68k");
         SekResetS68k();
+        SekCycleCntS68k += 40;
       }
       if (((dold & 3) == 1) != ((d & 3) == 1)) {
         elprintf(EL_INTSW, "m68k: s68k brq %i", d >> 1);
@@ -226,15 +233,15 @@ void m68k_reg_write8(u32 a, u32 d)
         remap_word_ram(d);
       goto write_comm;
     case 6:
-      Pico.rom[MEM_BE2(0x72)] = d; // simple hint vector changer
+      Pico_mcd->bios[MEM_BE2(0x72)] = d; // simple hint vector changer
       return;
     case 7:
-      Pico.rom[MEM_BE2(0x73)] = d;
+      Pico_mcd->bios[MEM_BE2(0x73)] = d;
       elprintf(EL_CDREGS, "hint vector set to %04x%04x",
-        ((u16 *)Pico.rom)[0x70/2], ((u16 *)Pico.rom)[0x72/2]);
+        ((u16 *)Pico_mcd->bios)[0x70/2], ((u16 *)Pico_mcd->bios)[0x72/2]);
       return;
     case 8:
-      (void) cdc_host_r(); // acts same as reading
+      (void) cdc_host_r(0); // acts same as reading
       return;
     case 0x0f:
       a = 0x0e;
@@ -334,7 +341,7 @@ u32 s68k_reg_read16(u32 a)
       d = cdc_reg_r();
       goto end;
     case 8:
-      d = cdc_host_r();
+      d = cdc_host_r(1);
       goto end;
     case 0xc:
       d = pcd_stopwatch_read(1);
@@ -363,8 +370,8 @@ u32 s68k_reg_read16(u32 a)
 
   d = (Pico_mcd->s68k_regs[a]<<8) | Pico_mcd->s68k_regs[a+1];
 
-  if (a >= 0x0e && a < 0x30)
-    return s68k_poll_detect(a, d);
+  if ((a >= 0x0e && a < 0x30) || a == 0x58)
+    d = s68k_poll_detect(a, d);
 
 end:
   return d;
@@ -456,9 +463,9 @@ void s68k_reg_write8(u32 a, u32 d)
     case 0x33: // IRQ mask
       elprintf(EL_CDREGS|EL_CD, "s68k irq mask: %02x", d);
       d &= 0x7e;
-      if ((d ^ Pico_mcd->s68k_regs[0x33]) & d & PCDS_IEN4) {
+      if ((d ^ Pico_mcd->s68k_regs[0x33]) & PCDS_IEN4) {
         // XXX: emulate pending irq instead?
-        if (Pico_mcd->s68k_regs[0x37] & 4) {
+        if ((d & PCDS_IEN4) && (Pico_mcd->s68k_regs[0x37] & 4)) {
           elprintf(EL_INTS, "cdd export irq 4 (unmask)");
           pcd_irq_s68k(4, 1);
         }
@@ -474,11 +481,8 @@ void s68k_reg_write8(u32 a, u32 d)
     case 0x37: {
       u32 d_old = Pico_mcd->s68k_regs[0x37];
       Pico_mcd->s68k_regs[0x37] = d & 7;
-      if ((d&4) && !(d_old&4)) {
-        // ??
-        pcd_event_schedule_s68k(PCD_EVENT_CDC, 12500000/75);
-
-        if (Pico_mcd->s68k_regs[0x33] & PCDS_IEN4) {
+      if ((d ^ d_old) & 4) {
+        if ((d & 4) && (Pico_mcd->s68k_regs[0x33] & PCDS_IEN4)) {
           elprintf(EL_INTS, "cdd export irq 4");
           pcd_irq_s68k(4, 1);
         }
@@ -552,7 +556,7 @@ void s68k_reg_write16(u32 a, u32 d)
       // these are only byte registers, LDS/UDS ignored
       return s68k_reg_write8(a + 1, d);
     case 0x08:
-      return (void) cdc_host_r(); // acts same as reading
+      return (void) cdc_host_r(1); // acts same as reading
     case 0x0a: // DMA address
       r[0xa] = d >> 8;
       r[0xb] = d;
@@ -665,29 +669,12 @@ static void PicoWriteM68k16_cell1(u32 a, u32 d)
 }
 #endif
 
-// BIOS faking for MSU-MD, checks for "SEGA" at 0x400100 to detect CD drive
-static u8 bios_id[4] = "SEGA";
-
-static u32 PicoReadM68k8_bios(u32 a)
-{
-  if ((a & 0xfffffc) == BASE+0x100) // CD detection by MSU
-    return bios_id[a&3];
-  return 0;
-}
-
-static u32 PicoReadM68k16_bios(u32 a)
-{
-  if ((a & 0xfffffc) == BASE+0x100) // CD detection by MSU
-    return (bios_id[a&2]<<8) | bios_id[(a&2)+1];
-  return 0;
-}
-
 // RAM cart (400000 - 7fffff, optional)
 static u32 PicoReadM68k8_ramc(u32 a)
 {
   u32 d = 0;
 
-  if (PicoIn.opt & POPT_EN_MCD_RAMCART) {
+  if (Pico.romsize == 0 && (PicoIn.opt & POPT_EN_MCD_RAMCART)) {
     if ((a & 0xf00001) == 0x400001) {
       if (Pico.sv.data != NULL)
         d = 3; // 64k cart
@@ -716,7 +703,7 @@ static u32 PicoReadM68k16_ramc(u32 a)
 
 static void PicoWriteM68k8_ramc(u32 a, u32 d)
 {
-  if (PicoIn.opt & POPT_EN_MCD_RAMCART) {
+  if (Pico.romsize == 0 && (PicoIn.opt & POPT_EN_MCD_RAMCART)) {
     if ((a & 0xf00001) == 0x600001) {
       if (Pico.sv.data != NULL && (Pico_mcd->m.bcram_reg & 1)) {
         Pico.sv.data[((a >> 1) & 0xffff) + 0x2000] = d;
@@ -783,7 +770,10 @@ void PicoWrite8_mcd_io(u32 a, u32 d)
     return;
   }
 
-  PicoWrite8_io(a, d);
+  if (carthw_ssf2_active)
+    carthw_ssf2_write8(a, d); // for MSU/MD+
+  else
+    PicoWrite8_io(a, d);
 }
 
 void PicoWrite16_mcd_io(u32 a, u32 d)
@@ -798,7 +788,10 @@ void PicoWrite16_mcd_io(u32 a, u32 d)
     return;
   }
 
-  PicoWrite16_io(a, d);
+  if (carthw_ssf2_active)
+    carthw_ssf2_write16(a, d); // for MSU/MD+
+  else
+    PicoWrite16_io(a, d);
 }
 #endif
 
@@ -991,7 +984,7 @@ static u32 PicoReadS68k8_pr(u32 a)
     a &= 0x1ff;
     if (a >= 0x0e && a < 0x30) {
       d = Pico_mcd->s68k_regs[a];
-      s68k_poll_detect(a & ~1, d);
+      d = s68k_poll_detect(a & ~1, d);
       goto regs_done;
     }
     d = s68k_reg_read16(a & ~1);
@@ -1214,7 +1207,7 @@ void pcd_state_loaded_mem(void)
   Pico_mcd->m.dmna_ret_2m &= 3;
 
   // restore hint vector
-  *(u16 *)(Pico.rom + 0x72) = Pico_mcd->m.hint_vector;
+  *(u16 *)(Pico_mcd->bios + 0x72) = Pico_mcd->m.hint_vector;
 }
 
 #ifdef EMU_M68K
@@ -1223,20 +1216,30 @@ static void m68k_mem_setup_cd(void);
 
 PICO_INTERNAL void PicoMemSetupCD(void)
 {
-  if (!Pico_mcd)
-    Pico_mcd = plat_mmap(0x05000000, sizeof(mcd_state), 0, 0);
-  memset(Pico_mcd, 0, sizeof(mcd_state));
-  pcd_base_address = (Pico.romsize > 0x20000 ? 0x400000 : 0x000000);
+  if (Pico_mcd == NULL) {
+    static u8 bios_id[4] = "SEGA";
+    PicoCreateMCD(NULL, 0);
+    // BIOS faking for MSU-MD, checks for "SEGA" at 0x400100 to detect CD drive
+    memcpy(Pico_mcd->bios+0x100, bios_id, 4);
+  }
+
+  pcd_base_address = (Pico.romsize ? 0x400000 : 0x000000);
 
   // setup default main68k map
   PicoMemSetup();
 
   // main68k map (BIOS or MSU mapped by PicoMemSetup()):
-  if (Pico.romsize > 0x20000) {
-    // MSU cartridge. Fake BIOS detection
-    cpu68k_map_set(m68k_read8_map,   0x400000, 0x41ffff, PicoReadM68k8_bios, 1);
-    cpu68k_map_set(m68k_read16_map,  0x400000, 0x41ffff, PicoReadM68k16_bios, 1);
-  } else {
+  cpu68k_map_set(m68k_read8_map,   BASE, BASE+0x01ffff, Pico_mcd->bios, 0);
+  cpu68k_map_set(m68k_read16_map,  BASE, BASE+0x01ffff, Pico_mcd->bios, 0);
+  if (pcd_base_address != 0) { // cartridge (for MSU/MD+)
+    // MD+ on MEGASD plus mirror
+    u32 base = 0x040000-(1<<M68K_MEM_SHIFT);
+    cpu68k_map_set(m68k_write8_map,  base, 0x03ffff, msd_write8, 1);
+    cpu68k_map_set(m68k_write16_map, base, 0x03ffff, msd_write16, 1);
+    cpu68k_map_set(m68k_write8_map,  base+0x080000, 0x0bffff, msd_write8, 1);
+    cpu68k_map_set(m68k_write16_map, base+0x080000, 0x0bffff, msd_write16, 1);
+    msd_reset();
+  } else { // no cartridge
     // RAM cart
     cpu68k_map_set(m68k_read8_map,   0x400000, 0x7fffff, PicoReadM68k8_ramc, 1);
     cpu68k_map_set(m68k_read16_map,  0x400000, 0x7fffff, PicoReadM68k16_ramc, 1);
