@@ -41,18 +41,26 @@ static void xmap_set(uptr *map, int shift, u32 start_addr, u32 end_addr,
     return;
   }
 
-  if (addr & 1) {
-    elprintf(EL_STATUS|EL_ANOMALY, "xmap_set: ptr is not aligned: %08lx", addr);
-    return;
+  if (is_func) {
+    if (addr & ((1 << MAP_FUNCTION_SHIFT) - 1)) {
+      elprintf(EL_STATUS|EL_ANOMALY, "xmap_set: function ptr is not aligned: %08lx", addr);
+      return;
+    }
+  } else {
+    if (addr & 1) {
+      elprintf(EL_STATUS|EL_ANOMALY, "xmap_set: data ptr is not aligned: %08lx", addr);
+      return;
+    }
   }
 
   if (!is_func)
     addr -= start_addr;
 
   for (i = start_addr >> shift; i <= end_addr >> shift; i++) {
-    map[i] = addr >> 1;
     if (is_func)
-      map[i] |= MAP_FLAG;
+      map[i] = (addr >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
+    else
+      map[i] = addr >> 1;
   }
 }
 
@@ -168,8 +176,8 @@ void cpu68k_map_read_funcs(u32 start_addr, u32 end_addr, u32 (*r8)(u32), u32 (*r
     r16map = s68k_read16_map;
   }
 
-  ar8 = (ar8 >> 1 ) | MAP_FLAG;
-  ar16 = (ar16 >> 1 ) | MAP_FLAG;
+  ar8 = (ar8 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
+  ar16 = (ar16 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
     r8map[i] = ar8, r16map[i] = ar16;
 }
@@ -194,10 +202,10 @@ void cpu68k_map_all_funcs(u32 start_addr, u32 end_addr, u32 (*r8)(u32), u32 (*r1
     w16map = s68k_write16_map;
   }
 
-  ar8 = (ar8 >> 1 ) | MAP_FLAG;
-  ar16 = (ar16 >> 1 ) | MAP_FLAG;
-  aw8 = (aw8 >> 1 ) | MAP_FLAG;
-  aw16 = (aw16 >> 1 ) | MAP_FLAG;
+  ar8 = (ar8 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
+  ar16 = (ar16 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
+  aw8 = (aw8 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
+  aw16 = (aw16 >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
     r8map[i] = ar8, r16map[i] = ar16, w8map[i] = aw8, w16map[i] = aw16;
 }
@@ -246,19 +254,19 @@ void m68k_map_unmap(u32 start_addr, u32 end_addr)
 
   addr = (uptr)m68k_unmapped_read8;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
-    m68k_read8_map[i] = (addr >> 1) | MAP_FLAG;
+    m68k_read8_map[i] = (addr >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
 
   addr = (uptr)m68k_unmapped_read16;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
-    m68k_read16_map[i] = (addr >> 1) | MAP_FLAG;
+    m68k_read16_map[i] = (addr >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
 
   addr = (uptr)m68k_unmapped_write8;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
-    m68k_write8_map[i] = (addr >> 1) | MAP_FLAG;
+    m68k_write8_map[i] = (addr >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
 
   addr = (uptr)m68k_unmapped_write16;
   for (i = start_addr >> shift; i <= end_addr >> shift; i++)
-    m68k_write16_map[i] = (addr >> 1) | MAP_FLAG;
+    m68k_write16_map[i] = (addr >> MAP_FUNCTION_SHIFT) | MAP_FLAG;
 }
 
 #ifndef _ASM_MEMORY_C
@@ -272,7 +280,7 @@ MAKE_68K_WRITE32(m68k_write32, m68k_write16_map)
 
 // -----------------------------------------------------------------
 
-static u32 ym2612_read_local_68k(void);
+static u32 ym2612_read_local_68k(u32 a);
 static int ym2612_write_local(u32 a, u32 d, int is_from_z80);
 static void z80_mem_setup(void);
 
@@ -922,7 +930,7 @@ static u32 PicoRead8_z80(u32 a)
   if ((a & 0x4000) == 0x0000) {
     d = PicoMem.zram[a & 0x1fff];
   } else if ((a & 0x6000) == 0x4000) // 0x4000-0x5fff
-    d = ym2612_read_local_68k();
+    d = ym2612_read_local_68k(a);
   else {
     elprintf(EL_UIO|EL_ANOMALY, "68k bad read [%06x] @%06x", a, SekPc);
     d = (u8)PicoRead16_floating(a);
@@ -1431,7 +1439,8 @@ static int ym2612_write_local(u32 a, u32 d, int is_from_z80)
           elprintf(EL_YMTIMER, "st mode %02x", d);
           ym2612_sync_timers(cycles, old_mode, d);
 
-          ym2612.OPN.ST.mode = d;
+          if ((d ^ old_mode) & 0xc0)
+            break;
 
           /* reset Timer a flag */
           if (d & 0x10)
@@ -1441,13 +1450,7 @@ static int ym2612_write_local(u32 a, u32 d, int is_from_z80)
           if (d & 0x20)
             ym2612.OPN.ST.status &= ~2;
 
-          if ((d ^ old_mode) & 0xc0) {
-#ifdef __GP2X__
-            if (PicoIn.opt & POPT_EXT_FM) return YM2612Write_940(a, d, get_scanline(is_from_z80));
-#endif
-            PsndDoFM(cycles);
-            return 1;
-          }
+          ym2612.OPN.ST.mode = d;
           return 0;
         }
         case 0x2a: { /* DAC data */
@@ -1477,28 +1480,40 @@ static int ym2612_write_local(u32 a, u32 d, int is_from_z80)
 }
 
 
-static u32 ym2612_read_local_z80(void)
+static u32 ym2612_read_local_z80(unsigned short a)
 {
   int xcycles = z80_cyclesDone() << 8;
 
   ym2612_update_status(xcycles);
 
-  elprintf(EL_YMTIMER, "timer z80 read %i, sched %i, %i @ %i|%i",
-    ym2612.OPN.ST.status, Pico.t.timer_a_next_oflow >> 8,
-    Pico.t.timer_b_next_oflow >> 8, xcycles >> 8, (xcycles >> 8) / 228);
-  return ym2612.OPN.ST.status;
+  if ((a&0x3) == 0 || !(PicoIn.opt & POPT_FM_YM2612)) {
+    elprintf(EL_YMTIMER, "timer z80 read %i, sched %i, %i @ %i|%i",
+      ym2612.OPN.ST.status, Pico.t.timer_a_next_oflow >> 8,
+      Pico.t.timer_b_next_oflow >> 8, xcycles >> 8, (xcycles >> 8) / 228);
+    Pico.t.ym2612_decay = xcycles + (OSC_NTSC/15/5<<8); // Q8 for convenience
+    return ym2612.OPN.ST.status_latch = ym2612.OPN.ST.status;
+  } else if (xcycles < Pico.t.ym2612_decay)
+    return ym2612.OPN.ST.status_latch;
+  else
+    return 0;
 }
 
-static u32 ym2612_read_local_68k(void)
+static u32 ym2612_read_local_68k(u32 a)
 {
   int xcycles = z80_cycles_from_68k() << 8;
 
   ym2612_update_status(xcycles);
 
-  elprintf(EL_YMTIMER, "timer 68k read %i, sched %i, %i @ %i|%i",
-    ym2612.OPN.ST.status, Pico.t.timer_a_next_oflow >> 8,
-    Pico.t.timer_b_next_oflow >> 8, xcycles >> 8, (xcycles >> 8) / 228);
-  return ym2612.OPN.ST.status;
+  if ((a&0x3) == 0 || !(PicoIn.opt & POPT_FM_YM2612)) {
+    elprintf(EL_YMTIMER, "timer 68k read %i, sched %i, %i @ %i|%i",
+      ym2612.OPN.ST.status, Pico.t.timer_a_next_oflow >> 8,
+      Pico.t.timer_b_next_oflow >> 8, xcycles >> 8, (xcycles >> 8) / 228);
+    Pico.t.ym2612_decay = xcycles + (OSC_NTSC/15/5<<8); // Q8 for convenience
+    return ym2612.OPN.ST.status_latch = ym2612.OPN.ST.status;
+  } else if (xcycles < Pico.t.ym2612_decay)
+    return ym2612.OPN.ST.status_latch;
+  else
+    return 0;
 }
 
 // legacy code, only used for GP2X
@@ -1526,11 +1541,13 @@ void ym2612_pack_state_old(void)
 int ym2612_pack_timers(void *buf, size_t size)
 {
   // timers are saved as tick counts, in 16.16 int format
-  int tac, tat = 0, tbc, tbt = 0, busy = 0;
+  int tac, tat = 0, tbc, tbt = 0, busy = 0, decay = 0;
   size_t b = 0;
 
   tac = 1024 - ym2612.OPN.ST.TA;
   tbc = 256  - ym2612.OPN.ST.TB;
+  if (Pico.t.ym2612_decay > 0)
+    decay = cycles_z80_to_68k(Pico.t.ym2612_decay);
   if (Pico.t.ym2612_busy > 0)
     busy = cycles_z80_to_68k(Pico.t.ym2612_busy);
   if (Pico.t.timer_a_next_oflow != TIMER_NO_OFLOW)
@@ -1540,12 +1557,14 @@ int ym2612_pack_timers(void *buf, size_t size)
   elprintf(EL_YMTIMER, "save: timer a %i/%i", tat >> 16, tac);
   elprintf(EL_YMTIMER, "save: timer b %i/%i", tbt >> 16, tbc);
 
-  assert(size >= 16);
+  assert(size >= 21);
   save_u16(buf, &b, ym2612.OPN.ST.TA);
   save_u16(buf, &b, ym2612.OPN.ST.TB);
   save_u32(buf, &b, tat);
   save_u32(buf, &b, tbt);
   save_u32(buf, &b, busy);
+  save_u32(buf, &b, decay);
+  save_u8_(buf, &b, ym2612.OPN.ST.status_latch);
   return b;
 }
 
@@ -1602,7 +1621,7 @@ void ym2612_unpack_state_old(void)
 
 void ym2612_unpack_timers(const void *buf, size_t size)
 {
-  int tac, tat, tbc, tbt, busy;
+  int tac, tat, tbc, tbt, busy, decay;
   size_t b = 0;
   assert(size >= 16);
   if (size < 16)
@@ -1615,6 +1634,8 @@ void ym2612_unpack_timers(const void *buf, size_t size)
   Pico.t.ym2612_busy = cycles_68k_to_z80(busy);
   tac = (1024 - ym2612.OPN.ST.TA) << 16;
   tbc = (256  - ym2612.OPN.ST.TB) << 16;
+  Pico.t.timer_a_step = TIMER_A_TICK_ZCYCLES * (1024 - ym2612.OPN.ST.TA);
+  Pico.t.timer_b_step = TIMER_B_TICK_ZCYCLES * (256  - ym2612.OPN.ST.TB);
   if (ym2612.OPN.ST.mode & 1)
     Pico.t.timer_a_next_oflow = (1LL * (tac-tat) * TIMER_A_TICK_ZCYCLES)>>16;
   else
@@ -1625,6 +1646,11 @@ void ym2612_unpack_timers(const void *buf, size_t size)
     Pico.t.timer_b_next_oflow = TIMER_NO_OFLOW;
   elprintf(EL_YMTIMER, "load: %i/%i, timer_a_next_oflow %i", tat>>16, tac>>16, Pico.t.timer_a_next_oflow >> 8);
   elprintf(EL_YMTIMER, "load: %i/%i, timer_b_next_oflow %i", tbt>>16, tbc>>16, Pico.t.timer_b_next_oflow >> 8);
+  if (size >= 21) {
+    decay = load_u32(buf, &b);
+    Pico.t.ym2612_decay = cycles_68k_to_z80(decay);
+    ym2612.OPN.ST.status_latch = load_u8_(buf, &b);
+  }
 }
 
 #if defined(NO_32X) && defined(_ASM_MEMORY_C)
