@@ -29,7 +29,8 @@ static s32 PsndBuffer[2*(54000+100)/50+2];
 s16 cdda_out_buffer[2*1152];
 
 // FM resampling polyphase FIR
-static resampler_t *fmresampler;
+static resampler_t *ym2612_resampler;
+static resampler_t *ym2413_resampler;
 static int (*PsndFMUpdate)(s32 *buffer, int length, int stereo, int is_buf_empty);
 
 PICO_INTERNAL void PsndInit(void)
@@ -41,10 +42,12 @@ PICO_INTERNAL void PsndInit(void)
 
 PICO_INTERNAL void PsndExit(void)
 {
-  OPLL_delete(opll);
+  if (opll)
+    OPLL_delete(opll);
   opll = NULL;
 
-  resampler_free(fmresampler); fmresampler = NULL;
+  resampler_free(ym2612_resampler); ym2612_resampler = NULL;
+  resampler_free(ym2413_resampler); ym2413_resampler = NULL;
 }
 
 PICO_INTERNAL void PsndReset(void)
@@ -66,7 +69,7 @@ static void YM2612Update(s32 *buffer, int length, int stereo)
 
 static int YM2612UpdateFIR(s32 *buffer, int length, int stereo, int is_buf_empty)
 {
-  resampler_update(fmresampler, buffer, length, YM2612Update);
+  resampler_update(ym2612_resampler, buffer, length, YM2612Update);
   return ymchans;
 }
 
@@ -83,12 +86,12 @@ static void YM2413Update(s32 *buffer, int length, int stereo)
 static int YM2413UpdateFIR(s32 *buffer, int length, int stereo, int is_buf_empty)
 {
   if (!is_buf_empty) memset(buffer, 0, (length << stereo) * sizeof(*buffer));
-  resampler_update(fmresampler, buffer, length, YM2413Update);
+  resampler_update(ym2413_resampler, buffer, length, YM2413Update);
   return 0;
 }
 
 // FIR setup, looks for a close enough rational number matching the ratio
-static void YMFM_setup_FIR(int inrate, int outrate, int stereo)
+static resampler_t *YMFM_setup_FIR(int inrate, int outrate, int stereo)
 {
   int mindiff = 999;
   int diff, mul, div;
@@ -108,8 +111,7 @@ static void YMFM_setup_FIR(int inrate, int outrate, int stereo)
   printf("FM polyphase FIR ratio=%d/%d error=%.3f%%\n",
         Pico.snd.fm_fir_mul, Pico.snd.fm_fir_div, 100.0*mindiff/inrate);
 
-  resampler_free(fmresampler);
-  fmresampler = resampler_new(FMFIR_TAPS, Pico.snd.fm_fir_mul, Pico.snd.fm_fir_div,
+  return resampler_new(FMFIR_TAPS, Pico.snd.fm_fir_mul, Pico.snd.fm_fir_div,
         0.85, 2, 2*inrate/50, stereo);
 }
 
@@ -137,11 +139,11 @@ void PsndRerate(int preserve_state)
   int state_size = 4096;
 
   // don't init YM2612 if preserve_state and no parameter changes
-  ym2612_init |= ymclock != ym2612_clock || ymopts != (PicoIn.opt & (POPT_DIS_FM_SSGEG|POPT_EN_FM_DAC));
+  ym2612_init |= ymclock != ym2612_clock || ymopts != (PicoIn.opt & (POPT_DIS_FM_SSGEG|POPT_FM_YM2612));
   ym2612_init |= ymrate != (PicoIn.opt & POPT_EN_FM_FILTER ? ym2612_rate : PicoIn.sndRate);
   ymclock = ym2612_clock;
   ymrate = (PicoIn.opt & POPT_EN_FM_FILTER ? ym2612_rate : PicoIn.sndRate);
-  ymopts = PicoIn.opt & (POPT_DIS_FM_SSGEG|POPT_EN_FM_DAC);
+  ymopts = PicoIn.opt & (POPT_DIS_FM_SSGEG|POPT_FM_YM2612);
 
   if (preserve_state && ym2612_init) {
     state = malloc(state_size);
@@ -149,25 +151,30 @@ void PsndRerate(int preserve_state)
       state_size = YM2612PicoStateSave3(state, state_size);
   }
 
-  if (PicoIn.AHW & PAHW_SMS) {
+  if (opll && opll->rate != ym2413_rate) {
     OPLL_setRate(opll, ym2413_rate);
     if (!preserve_state)
       OPLL_reset(opll);
-    YMFM_setup_FIR(ym2413_rate, PicoIn.sndRate, 0);
+    resampler_free(ym2413_resampler);
+    ym2413_resampler = YMFM_setup_FIR(ym2413_rate, PicoIn.sndRate, 0);
+  }
+  if (PicoIn.AHW & PAHW_SMS) {
     PsndFMUpdate = YM2413UpdateFIR;
   } else if ((PicoIn.opt & POPT_EN_FM_FILTER) && ym2612_rate != PicoIn.sndRate) {
     // polyphase FIR resampler, resampling directly from native to output rate
     if (ym2612_init)
       YM2612Init(ym2612_clock, ym2612_rate,
         ((PicoIn.opt&POPT_DIS_FM_SSGEG) ? 0 : ST_SSG) |
-        ((PicoIn.opt&POPT_EN_FM_DAC)    ? ST_DAC : 0));
-    YMFM_setup_FIR(ym2612_rate, PicoIn.sndRate, PicoIn.opt & POPT_EN_STEREO);
+        ((PicoIn.opt&POPT_FM_YM2612)    ? ST_DAC : 0));
+    resampler_free(ym2612_resampler);
+    ym2612_resampler = YMFM_setup_FIR(ym2612_rate, PicoIn.sndRate,
+        PicoIn.opt & POPT_EN_STEREO);
     PsndFMUpdate = YM2612UpdateFIR;
   } else {
     if (ym2612_init)
       YM2612Init(ym2612_clock, PicoIn.sndRate,
         ((PicoIn.opt&POPT_DIS_FM_SSGEG) ? 0 : ST_SSG) |
-        ((PicoIn.opt&POPT_EN_FM_DAC)    ? ST_DAC : 0));
+        ((PicoIn.opt&POPT_FM_YM2612)    ? ST_DAC : 0));
     PsndFMUpdate = YM2612UpdateONE;
   }
 
@@ -320,7 +327,7 @@ PICO_INTERNAL void PsndDoSMSFM(int cyc_to)
 
   if (Pico.m.hardware & PMS_HW_FMUSED) {
     buf += pos;
-    PsndFMUpdate(buf32, len, 0, 0);
+    YM2413UpdateFIR(buf32, len, 0, 0);
     if (stereo) 
       while (len--) {
         *buf++ += *buf32;
@@ -431,6 +438,10 @@ void cdda_start_play(int lba_base, int lba_offset, int lb_len)
 
     mp3_start_play(Pico_mcd->cdda_stream, pos1024);
     return;
+  } else if (Pico_mcd->cdda_type == CT_OGG) {
+    ogg_start_play(Pico_mcd->cdda_stream,
+                  lba_offset * 588 + Pico_mcd->m.cdda_lba_offset);
+    return;
   }
 
   // on restart after loading, consider offset of last played sample
@@ -443,6 +454,13 @@ void cdda_start_play(int lba_base, int lba_offset, int lb_len)
   }
 }
 
+void cdda_stop_play(void)
+{
+  if (Pico_mcd->cdda_type == CT_OGG)
+    ogg_stop_play();
+  Pico_mcd->cdda_stream = NULL;
+}
+
 
 PICO_INTERNAL void PsndClear(void)
 {
@@ -536,6 +554,8 @@ static int PsndRender(int offset, int length)
   {
     if (Pico_mcd->cdda_type == CT_MP3)
       mp3_update(buf32, length-offset, stereo);
+    else if (Pico_mcd->cdda_type == CT_OGG)
+      ogg_update(buf32, length-offset, stereo);
     else
       cdda_raw_update(buf32, length-offset, stereo);
   }
