@@ -11,12 +11,11 @@
 
 #include <cpu/sh2/sh2.h>
 #include "sound/ym2612.h"
-#include "sound/emu2413/emu2413.h"
+#include "sound/ym2413.h"
+#include "sound/sn76496.h"
+#include "cd/megasd.h"
 #include "state.h"
 
-// sn76496 & ym2413
-extern int *sn76496_regs;
-
 static arearw    *areaRead;
 static arearw    *areaWrite;
 static areaeof   *areaEof;
@@ -107,7 +106,7 @@ typedef enum {
   CHUNK_RC,      // old
   CHUNK_MISC_CD,
   //
-  CHUNK_IOPORTS, // versions < 1.70 did not save that..
+  CHUNK_IOPORTS, // old
   CHUNK_SMS,     // 25
   // 32x
   CHUNK_MSH2,
@@ -135,6 +134,11 @@ typedef enum {
   CHUNK_YM2413,
   CHUNK_PICO_PCM,
   CHUNK_PICO,
+  CHUNK_CD_MSD,
+  CHUNK_VDP,
+  CHUNK_FM_TIMERS,
+  CHUNK_FMv3 = 60,
+  CHUNK_IOPORTSv2,
   //
   CHUNK_DEFAULT_COUNT,
   CHUNK_CARTHW_ = CHUNK_CARTHW,  // 64 (defined in PicoInt)
@@ -225,7 +229,6 @@ static int state_save(void *file)
 {
   char sbuff[32] = "Saving.. ";
   unsigned char buff[0x60], buff_z80[Z80_STATE_SIZE];
-  void *ym_regs = YM2612GetRegs();
   void *buf2 = NULL;
   int ver = 0x0191; // not really used..
   int retval = -1;
@@ -248,14 +251,24 @@ static int state_save(void *file)
     CHECKED_WRITE_BUFF(CHUNK_RAM,   PicoMem.ram);
     CHECKED_WRITE_BUFF(CHUNK_VSRAM, PicoMem.vsram);
     CHECKED_WRITE_BUFF(CHUNK_IOPORTS, PicoMem.ioports);
+    len = io_ports_pack(buf2, CHUNK_LIMIT_W);
+    CHECKED_WRITE(CHUNK_IOPORTSv2, len, buf2);
     if (PicoIn.AHW & PAHW_PICO) {
       len = PicoPicoPCMSave(buf2, CHUNK_LIMIT_W);
       CHECKED_WRITE(CHUNK_PICO_PCM, len, buf2);
       CHECKED_WRITE(CHUNK_PICO, sizeof(PicoPicohw), &PicoPicohw);
     } else {
-      ym2612_pack_state();
-      ym_regs = YM2612GetRegs();
+#ifdef __GP2X__
+      void *ym_regs = YM2612GetRegs();
+      ym2612_pack_state_old();
       CHECKED_WRITE(CHUNK_FM, 0x200+4, ym_regs);
+#else
+      // write fm state first since timer load needs OPN.ST.mode
+      len = YM2612PicoStateSave3(buf2, CHUNK_LIMIT_W);
+      CHECKED_WRITE(CHUNK_FMv3, len, buf2);
+      len = ym2612_pack_timers(buf2, CHUNK_LIMIT_W);
+      CHECKED_WRITE(CHUNK_FM_TIMERS, len, buf2);
+#endif
     }
 
     if (!(PicoIn.opt & POPT_DIS_IDLE_DET))
@@ -263,8 +276,11 @@ static int state_save(void *file)
   }
   else {
     CHECKED_WRITE_BUFF(CHUNK_SMS, Pico.ms);
-    ym_regs = YM2413GetRegs();
-    CHECKED_WRITE(CHUNK_YM2413, 0x40+4, ym_regs);
+    // only store the FM unit state if it was really used
+    if (Pico.m.hardware & PMS_HW_FMUSED) {
+      len = ym2413_pack_state(buf2, CHUNK_LIMIT_W);
+      CHECKED_WRITE(CHUNK_YM2413, len, buf2);
+    }
   }
   CHECKED_WRITE(CHUNK_PSG, 28*4, sn76496_regs);
 
@@ -278,7 +294,8 @@ static int state_save(void *file)
   CHECKED_WRITE_BUFF(CHUNK_CRAM,  PicoMem.cram);
 
   CHECKED_WRITE_BUFF(CHUNK_MISC,  Pico.m);
-  PicoVideoSave();
+  len = PicoVideoSave(buf2);
+  CHECKED_WRITE(CHUNK_VDP, len, buf2);
   CHECKED_WRITE_BUFF(CHUNK_VIDEO, Pico.video);
 
   if (PicoIn.AHW & PAHW_MCD)
@@ -287,7 +304,7 @@ static int state_save(void *file)
     SekPackCpu(buff, 1);
     if (Pico_mcd->s68k_regs[3] & 4) // 1M mode?
       wram_1M_to_2M(Pico_mcd->word_ram2M);
-    memcpy(&Pico_mcd->m.hint_vector, Pico.rom + 0x72,
+    memcpy(&Pico_mcd->m.hint_vector, Pico_mcd->bios + 0x72,
       sizeof(Pico_mcd->m.hint_vector));
 
     CHECKED_WRITE_BUFF(CHUNK_S68K,     buff);
@@ -309,6 +326,8 @@ static int state_save(void *file)
     len = cdd_context_save(buf2);
     CHECKED_WRITE(CHUNK_CD_CDD, len, buf2);
 
+    CHECKED_WRITE_BUFF(CHUNK_CD_MSD, Pico_msd);
+
     if (Pico_mcd->s68k_regs[3] & 4) // convert back
       wram_2M_to_1M(Pico_mcd->word_ram2M);
   }
@@ -403,13 +422,15 @@ static int state_load(void *file)
   unsigned char buff_m68k[0x60], buff_s68k[0x60];
   unsigned char buff_z80[Z80_STATE_SIZE];
   unsigned char buff_sh2[SH2_STATE_SIZE];
+  unsigned char buff_vdp[0x200];
   unsigned char *buf = NULL;
   unsigned char chunk;
   void *ym_regs;
   int len_check;
   int retval = -1;
   char header[8];
-  int ver, len;
+  int ver, has_32x = 0, has_iov2 = 0;
+  int len, len_vdp = 0;
 
   memset(buff_m68k, 0, sizeof(buff_m68k));
   memset(buff_s68k, 0, sizeof(buff_s68k));
@@ -436,7 +457,10 @@ static int state_load(void *file)
     if (len < 0 || len > 1024*512) R_ERROR_RETURN("bad length");
     if (CHUNK_S68K <= chunk && chunk <= CHUNK_MISC_CD && !(PicoIn.AHW & PAHW_MCD))
       R_ERROR_RETURN("cd chunk in non CD state?");
-    if (CHUNK_32X_FIRST <= chunk && chunk <= CHUNK_32X_LAST && !(PicoIn.AHW & PAHW_32X))
+
+    // 32X only appears in PicoDrive after it has been enabled, so track this
+    has_32x |= CHUNK_32X_FIRST <= chunk && chunk <= CHUNK_32X_LAST;
+    if (has_32x && !(PicoIn.AHW & PAHW_32X))
       Pico32xStartup();
 
     switch (chunk)
@@ -455,23 +479,30 @@ static int state_load(void *file)
       case CHUNK_CRAM:    CHECKED_READ_BUFF(PicoMem.cram); break;
       case CHUNK_VSRAM:   CHECKED_READ_BUFF(PicoMem.vsram); break;
       case CHUNK_MISC:    CHECKED_READ_BUFF(Pico.m); break;
-      case CHUNK_VIDEO:
-        CHECKED_READ_BUFF(Pico.video);
-        PicoVideoLoad();
-        break;
+      case CHUNK_VIDEO:   CHECKED_READ_BUFF(Pico.video); break;
+      case CHUNK_VDP:     CHECKED_READ2((len_vdp = len), buff_vdp); break;
 
       case CHUNK_IOPORTS: CHECKED_READ_BUFF(PicoMem.ioports); break;
+      case CHUNK_IOPORTSv2:
+        CHECKED_READ(len, buf);
+        io_ports_unpack(buf, len);
+        has_iov2 = 1;
+        break;
+
+
       case CHUNK_PSG:     CHECKED_READ2(28*4, sn76496_regs); break;
       case CHUNK_YM2413:
-        ym_regs = YM2413GetRegs();
-        CHECKED_READ2(0x40+4, ym_regs);
-        YM2413UnpackState();
+        CHECKED_READ(len, buf);
+        ym2413_unpack_state(buf, len);
+        Pico.m.hardware |= PMS_HW_FMUSED;
         break;
       case CHUNK_FM:
         ym_regs = YM2612GetRegs();
         CHECKED_READ2(0x200+4, ym_regs);
-        ym2612_unpack_state();
+        ym2612_unpack_state_old();
         break;
+      case CHUNK_FM_TIMERS: CHECKED_READ(len, buf); ym2612_unpack_timers(buf, len); break;
+      case CHUNK_FMv3:      CHECKED_READ(len, buf); YM2612PicoStateLoad3(buf, len); break;
 
       case CHUNK_PICO_PCM:
         CHECKED_READ(len, buf);
@@ -497,6 +528,7 @@ static int state_load(void *file)
       case CHUNK_GA_REGS:  CHECKED_READ_BUFF(Pico_mcd->s68k_regs); break;
       case CHUNK_PCM:      CHECKED_READ_BUFF(Pico_mcd->pcm); break;
       case CHUNK_MISC_CD:  CHECKED_READ_BUFF(Pico_mcd->m); break;
+      case CHUNK_CD_MSD:   CHECKED_READ_BUFF(Pico_msd); break;
 
       case CHUNK_CD_EVT:
         CHECKED_READ2(0x40, buf);
@@ -582,6 +614,12 @@ breakswitch:
   }
 
 readend:
+  PicoVideoLoad(buff_vdp, len_vdp);
+
+  if (PicoIn.AHW & PAHW_32X)
+    if (!has_32x)
+      Pico32xShutdown(); // in case of loading a state with 32X disabled
+
   if (PicoIn.AHW & PAHW_SMS)
     PicoStateLoadedMS();
 
@@ -599,8 +637,6 @@ readend:
 
   z80_unpack(buff_z80);
 
-  // due to dep from 68k cycles..
-  Pico.t.m68c_frame_start = Pico.t.m68c_aim;
   if (PicoIn.AHW & PAHW_32X)
     Pico32xStateLoaded(0);
   if (PicoIn.AHW & PAHW_MCD)
@@ -611,6 +647,10 @@ readend:
     Pico.video.status |= (Pico.video.pending_ints << 2) & SR_F;
   }
 
+  // kludge: if there is no new ports data save, they must be initialised
+  if (!has_iov2)
+    io_ports_reset();
+
   Pico.m.dirtyPal = 1;
   retval = 0;
 
@@ -622,6 +662,8 @@ out:
 static int state_load_gfx(void *file)
 {
   int ver, len, found = 0, to_find = 4;
+  u8 buff_vdp[0x200];
+  int len_vdp = 0;
   char buff[8];
 
   if (PicoIn.AHW & PAHW_32X)
@@ -647,6 +689,7 @@ static int state_load_gfx(void *file)
       case CHUNK_CRAM:  CHECKED_READ_BUFF(PicoMem.cram);  found++; break;
       case CHUNK_VSRAM: CHECKED_READ_BUFF(PicoMem.vsram); found++; break;
       case CHUNK_VIDEO: CHECKED_READ_BUFF(Pico.video); found++; break;
+      case CHUNK_VDP:   CHECKED_READ2((len_vdp = len), buff_vdp); break;
 
 #ifndef NO_32X
       case CHUNK_DRAM:
@@ -672,6 +715,7 @@ static int state_load_gfx(void *file)
         break;
     }
   }
+  PicoVideoLoad(buff_vdp, len_vdp);
 
 out:
 readend:
@@ -735,10 +779,10 @@ int PicoStateLoadGfx(const char *fname)
     areaRead(PicoMem.vsram, 1, sizeof(PicoMem.vsram), afile);
     areaSeek(afile, 0x221a0, SEEK_SET);
     areaRead(&Pico.video, 1, sizeof(Pico.video), afile);
+    PicoVideoCacheSAT(1);
   }
   areaClose(afile);
 
-  PicoVideoCacheSAT(1);
   Pico.est.rendstatus = -1;
   return 0;
 }
@@ -749,10 +793,12 @@ struct PicoTmp
   unsigned short vram[0x8000];
   unsigned short cram[0x40];
   unsigned short vsram[0x40];
-  unsigned int satcache[0x80];
+  unsigned int satcache[2*0x80];
 
   //struct PicoMisc m;
   struct PicoVideo video;
+  u8 vdp[0x200];
+  int vdp_len;
 
   struct {
     struct Pico32x p32x;
@@ -774,6 +820,7 @@ void *PicoTmpStateSave(void)
   memcpy(t->vsram, PicoMem.vsram, sizeof(PicoMem.vsram));
   memcpy(t->satcache, VdpSATCache, sizeof(VdpSATCache));
   memcpy(&t->video, &Pico.video, sizeof(Pico.video));
+  t->vdp_len = PicoVideoSave(t->vdp);
 
 #ifndef NO_32X
   if (PicoIn.AHW & PAHW_32X) {
@@ -798,7 +845,7 @@ void PicoTmpStateRestore(void *data)
   memcpy(VdpSATCache, t->satcache, sizeof(VdpSATCache));
   memcpy(&Pico.video, &t->video, sizeof(Pico.video));
   Pico.m.dirtyPal = 1;
-  PicoVideoCacheSAT(0);
+  PicoVideoLoad(t->vdp, t->vdp_len);
 
 #ifndef NO_32X
   if (PicoIn.AHW & PAHW_32X) {
