@@ -630,6 +630,8 @@ void lprintf(const char *fmt, ...)
 /* libretro */
 bool libretro_supports_bitmasks = false;
 
+#define EXTENSIONS "bin|gen|smd|md|32x|cue|iso|chd|m3u|sms|gg|sg|sc|68k|sgd|pco|vgm|vgz"
+
 void retro_set_environment(retro_environment_t cb)
 {
    bool option_categories_supported;
@@ -639,7 +641,7 @@ void retro_set_environment(retro_environment_t cb)
 
    static const struct retro_system_content_info_override content_overrides[] = {
       {
-         "bin|gen|smd|md|32x|sms|gg|sg|sc|68k|sgd|pco", /* extensions */
+         EXTENSIONS,
 #if defined(LOW_MEMORY)
          true,                         /* need_fullpath */
 #else
@@ -685,7 +687,7 @@ void retro_get_system_info(struct retro_system_info *info)
    memset(info, 0, sizeof(*info));
    info->library_name = "PicoDrive";
    info->library_version = VERSION;
-   info->valid_extensions = "bin|gen|smd|md|32x|cue|iso|chd|sms|gg|sg|sc|m3u|68k|sgd|pco";
+   info->valid_extensions = EXTENSIONS;
    info->need_fullpath = true;
 }
 
@@ -1997,12 +1999,12 @@ static void update_variables(bool first_run)
 #endif
 
    var.value = NULL;
-   var.key = "picodrive_dacnoise";
+   var.key = "picodrive_fmchip";
    if (environ_cb(RETRO_ENVIRONMENT_GET_VARIABLE, &var) && var.value) {
-      if (strcmp(var.value, "enabled") == 0)
-         PicoIn.opt |= POPT_EN_FM_DAC;
+      if (strcmp(var.value, "ym2612") == 0)
+         PicoIn.opt |= POPT_FM_YM2612;
       else
-         PicoIn.opt &= ~POPT_EN_FM_DAC;
+         PicoIn.opt &= ~POPT_FM_YM2612;
    }
 
    var.value = NULL;
